#
# September 2024 - Gass Daniel - https://dangasdiy.top/
#
# Off-road vehicle driving 
#
from time import sleep
from L293D_lib import L293D_motor, servo, IR_read, sendmsg 
from sys import exit
from machine import Pin
# motor
def move(val99, dur99):
    motor_front.setSpeed(val99,val99)
    motor_rear.setSpeed(val99,val99)
    sleep(dur99)
    motor_front.setSpeed(0,0)
    motor_rear.setSpeed(0,0)
# servos
def monte_av(val98,fb):
    for i in range (1, val98, 1):
        sleep(sleeptime)
        servo_front.move(fb)
def monte_ar(val98,fb):
    for i in range (1, val98, 1):
        sleep(sleeptime)
        servo_rear.move(fb)
# IR
def detect() : # detect obstacle
        ir_read_tab = IR_read()
        val = 30
        motor_front.setSpeed(val,val)
        motor_rear.setSpeed(val,val)
        while ir_read_tab[0] == 90 :
            ir_read_tab = IR_read()
        sleep(0.3)
        ir_read_tab = IR_read()
        motor_front.setSpeed(0,0)
        motor_rear.setSpeed(0,0)  
        buf =  " Detection distance : " + str(ir_read_tab[4]) + " angles : " + str(ir_read_tab[0]) + " " + str(ir_read_tab[1]) + " " + str(ir_read_tab[2]) + " " + str(ir_read_tab[3])
        sendmsg(buf)
        
# motor_rear declaration  (left rear wheel (ena,in1) right rear wheel (enb,in2)
motor_rear  = L293D_motor(12, 13, 10, 11, "rear")
# motor_front declaration (left-hand wheel (ena,in1) right-hand front wheel (enb,in2)
motor_front = L293D_motor(19, 18, 21, 20, "front")
# servo front declaration (pin left servo , rest, min, max, pin right servo, rest, min, max) values in servo angles
servo_front = servo(16, 90, 34, 146, 17, 90, 34, 146, "front")
# servo rear declaration (pin left servo , rest, min, max, pin right servo, rest, min, max) values in servo angles
servo_rear = servo(15, 90, 46, 134, 14, 90, 46, 134, "rear")
# for bluetooth
from machine import UART, ADC
import time
uart = UART(0, 9600)

# init some features
led_int = Pin("LED", Pin.OUT)
co = 1           # display counter
incr = 1         # angle unit for servo motor movement 
sleeptime = 0.05 # time between two servomotor commands
hight = 0        # hight of obstacle
# init servo in rest position
servo_front.move(0)
servo_rear.move(0)
sendmsg("==========================> Ready to start - just wait for bluetooth online <=======================")
try:
    while True:
        val1 = 0   # joystick up/down
        val2 = 0   # joystick left/right
        val3 = '?' # used button
        if uart.any() > 0: # waiting for info from remote control
            data = uart.read()
            # extracting information received
#            print(co, " Data received: ",data) ; co += 1
            ext = data.split() ; exc = len(ext)
            if exc > 1 : val1 = int(ext[0]) ; val2 = int(ext[1])
            if exc == 3 : val3 = ext[2]
#            print(co, " Data split ", val1, val2, val3); co += 1

            # motor stops engines
            if val1 == 0 and val2 == 0  : motor_rear.setSpeed(0,0) ; motor_front.setSpeed(0,0)

            # ================   move vehicle forward or backward
            
            if val3 == '?' and val1 != 0 : motor_rear.setSpeed(val1,val1); motor_front.setSpeed(val1,val1)

            # ================   read hight of obstacle
            
            elif val3 == b'3' :
                ir_read_tab = IR_read()
                buf =  " Detection distance : " + str(ir_read_tab[4]) + " angles : " + str(ir_read_tab[0]) + " " + str(ir_read_tab[1]) + " " + str(ir_read_tab[2]) + " " + str(ir_read_tab[3])
                sendmsg(buf) 
            
            # ================   climb down the obstacle
            
            elif val3 == b'4' :
                # front wheel locks
                trv0 = 111 - 90
                monte_av(trv0,1)
                # advance to overrun
                move(35,1)
                # front wheels down
                servo_front.close()
                monte_av(90-86,-1)
                # monte_av(90-62,-1)
                monte_ar(90-68,-1) # bloque roues ar
                # step forward
                move(30,1)
                servo_front.close()
                # front wheel locks
                monte_av(117 - 90, 1)
                servo_rear.close()
                monte_ar(134 - 90, 1) # descend roue ar
                # step forward
                move(30,1.5)              
                # rest
                servo_front.close()
                servo_rear.close()
     
            
            # ================   turn vehicle
            
            elif val3 == b'5':
                # move vehicle right
                if val2 > 0: motor_rear.setSpeed(-80,80) ; motor_front.setSpeed(-80,80)
                # move vehicle left
                if val2 < 0: motor_rear.setSpeed(80,-80) ; motor_front.setSpeed(80,-80)

            # ================   front wheels manual          
                
            elif val3 == b'1' : # mount the front wheels
                servo_front.move(incr)

            elif val3 == b'6' : # down front wheels
                servo_front.move(-incr)
       
            # ================   rear wheels manual
            
            elif val3 == b'2' : # mount rear wheels
                servo_rear.move(-incr)

            elif val3 == b'7' : # down the rear wheels
                servo_rear.move(incr)
            # ================  detect and climb an obstacle           
                
            elif val3 == b'8'  : # mount the front wheels
                detect()
                ir_read_tab = IR_read()
                if ir_read_tab[0] != 90 :
                    # mounts two front wheels 
                    hight = ir_read_tab[0] - 90
                    monte_av(hight,1)
                    # placing front wheels on the platform while moving forward
                    val = 30
                    motor_front.setSpeed(val,val)
                    motor_rear.setSpeed(val,val)
                    sleep(1)
                    servo_front.close()
                    # placing other front wheels on the platform while moving forward
                    hight = 90 - ir_read_tab[1]
                    monte_av(hight,-1)
                    # installs the 4 front wheels
                    move(40, 1.5)
                    servo_front.close()
                    # mounts rear wheels
                    hight = 90 - ir_read_tab[2]
                    monte_ar(hight,-1)
                    # placing rear wheels on the platform while moving forward
                    move(40, 1.5)
                    servo_rear.close() # puts servo at rest
                    # mount all the rear wheels on the rear axle while moving forward
                    hight = ir_read_tab[3] - 90
                    monte_ar(hight,1)
                    move(40, 1)
                    servo_rear.close()  # puts servo at rest
                                  
            # ================  puts at rest  

            elif val3 == b'9' : # application shutdown
                motor_rear.close()
                motor_front.close()
                servo_front.close() # puts servo at rest
                servo_rear.close()  # puts servo at rest
                exit()
                
            elif val3 == b'0' : # puts servos at rest
                servo_front.close() 
                servo_rear.close()  

                
except KeyboardInterrupt:
    print('Keyboard Interrupt')
    motor_rear.close()
    motor_front.close()
    servo_front.close()
    servo_rear.close()    